﻿
/************************************** PARAMETERS ********************************************/
# working period duration (minutes)
param PHI;

# set of mold types
set M;

# set of mold types + empty mold
set MExt := {0} union M;

# set of heaters
set H;

# set of parts
set Q;

# demand of mold type i
param dm{i in M} integer >=0;

# time of vulcanization for a mold type i in heater k
param tv{i in M, k in H}  >=0 , default 0;

# time needed to place mold i in a heater (setup time).
param tc{i in MExt}  >=0, default 0;

# time needed to remove mold i from a heater (removal time)
param tq{i in MExt}  >=0, default 0;

# numbers of type mold i
param nm{i in M} integer >=0;

# numbers of part q
param np{q in Q} integer >=0, default 0;

# binary parameter that indicates if mold i requires part q
param rp{i in M, q in Q} binary default 0;

# initial inventory of tires type i
param prdInit{i in M} integer default 0;

# number of periods of the planning horizon (THB)
param PeriodNumber  ;

# set of periods
set P :={1..PeriodNumber};
set PExt := {0} union P ;
set PIdx := {2..PeriodNumber};

# mold-heater combinations allowed
param C{ (i,k) in {M,H}} binary;
param CExt{ (i,k) in {MExt,H}} binary := if i == 0 then 1 else C[i,k];

# mold-mold combinations allowed
param MC{ (i,j) in {M,M} } binary ;
param MCExt{(i,j) in {MExt, M}} := if i ==0 then 1 else if i > j then 0 else MC[i,j];
set MCS:= {(i,j) in {M, M}: MC[i,j]};

# set of pair of molds (i,j) that can be cured in heater k, with i <= j
set T1 {i in M}:= {(j,k) in {M , H}: i<=j and MCExt[i,j] and CExt[i,k] and CExt[j,k]};

# set of pair of molds (i,j) that can be cured in heater k, with i >= j
set T2 {i in M}:= {(j,k) in {M  , H}: i>=j and MCExt[j,i] and  CExt[i,k] and CExt[j,k]};

# set of mold i that requires part q
set PC {q in Q} := { i in M : rp[i,q] };

param CONSTANT := 2*card(H);

/************************************** VARIABLES ********************************************/

# number of molds i used in heater k during period t, with x_{ikt} in {0,1,2}
var x {i in M, k in H, t in PExt} integer >= 0, <= 2;

# number of set-up of molds i in heater k during period t, with yc_{ikt} in {0,1,2}
var yc {i in M, k in H, t in P} integer >=0, <= 2;

# number of removal of molds i in heater k during period t, with yq_{ikt} in {0,1,2}
var yq {i in M, k in H, t in P} integer >=0, <= 2;

# binary variable that indicates if molds (i,j) are produced in heater k during period t
var z {(i, j, k, t) in {MExt, M, H, P}:  i<=j and MCExt[i,j] and CExt[i,k] and CExt[j,k]} binary;

# binary variable that indicates if at period t there exists some heater in use
var w {t in P} binary;

# number of tires cured using mold i when assigned jointly with mold j in heater k during period t
var u { (i, j, k, t) in {MExt, M, H, P}:  i<=j and MCExt[i,j] and CExt[i,k] and CExt[j,k]} integer >= 0;

# Production quantity of tires of the mold type i in period t
var prd {i in M } integer >= 0;


/************************************** MODEL ********************************************/
minimize makespan: sum{t in P } w[t];


s.t. r1 {t in PIdx}:

    w[t] <= w[t-1];

s.t. r2 {t in P}:

    CONSTANT * w[t] >= sum{(k,i,j) in {H, MExt, M}: MCExt[i,j] and CExt[i,k] and CExt[j,k]} z[i,j,k,t] ;

s.t. r3 {(k,t) in {H,P}}:

    sum{(i,j) in {MExt, M}: i<=j and MCExt[i,j] and CExt[i,k] and CExt[j,k]} z[i,j,k,t]  <= 1;

s.t. r4 {(i,k,t) in {M,H,P}: C[i,k]}:

    u[0,i,k,t] <= (PHI - yc[i,k,t]*tc[i] - sum{l in M: C[l,k]} yq[l,k,t] * tq[l]) / tv[i,k];

s.t. r5 {(i,j,k,t) in {M,M,H,P}: i<j and MC[i,j] and C[i,k] and C[j,k]}:

    u[i,j,k,t] <= (PHI - yc[i,k,t]*tc[i] - yc[j,k,t]*tc[j] - sum{l in M: C[l,k]} yq[l,k,t] * tq[l] ) / max( tv[i,k], tv[j,k] );

s.t. r6 {(i,k,t) in {M,H,P}: C[i,k]}:

    u[i,i,k,t] <= (PHI - yc[i,k,t]*tc[i] - sum{l in M: C[l,k]} yq[l,k,t] * tq[l] ) / tv[i,k];

s.t. r7 {(i,j,k,t) in {MExt, M, H, P}: i <= j and j > 0 and MCExt[i,j] and CExt[i,k] and CExt[j,k]}:

    u[i,j,k,t] <= PHI * z[i,j,k,t];

s.t. r8 {i in M} :  prdInit[i] + sum{t in P} 
                                    (
                                        (sum{(j,k) in {M,H}: i<=j and MC[i,j] and C[i,k] and C[j,k]} u[i,j,k,t]) + 
                                        (sum{(j,k) in {MExt,H}: i>=j and MCExt[j,i] and CExt[i,k] and CExt[j,k]} u[j,i,k,t])
                                    ) >= dm[i];

s.t. r8b {i in M} :  prdInit[i] + sum{t in P} 
                                    (
                                        (sum{(j,k) in {M,H}: i<=j and MC[i,j] and C[i,k] and C[j,k]} u[i,j,k,t]) + 
                                        (sum{(j,k) in {MExt,H}: i>=j and MCExt[j,i] and CExt[i,k] and CExt[j,k]} u[j,i,k,t])
                                    ) = prd[i];

s.t. r9 {(i,k,t) in {M, H, P} : C[i,k]}:

    x[i,k,t] = sum{j in M: i<=j and MC[i,j] and C[j,k]} z[i,j,k,t] +
               sum{j in MExt: i >= j and MCExt[j,i] and CExt[j,k] } z[j,i,k,t];

s.t. r10 {(i,t) in {M, P}} : sum{ k in H } x[i,k,t] <= nm[i];

s.t. r10p {(i,t) in {M, P}} : sum{ k in H } x[i,k,0] = 0;

s.t. r11 {(q,t) in {Q,P}} : sum{(i,k) in {PC[q], H}} x[i,k,t] <= np[q];

s.t. r12 {(i,k,t) in {M, H, P }: C[i,k]} : yc[i,k,t] >= x[i,k,t] - x[i,k,t-1];

s.t. r13 {(i,k,t) in {M, H, PIdx }: C[i,k]} : yq[i,k,t] >= x[i,k,t-1] - x[i,k,t];

s.t. r14 {(i,k) in {M, H }: C[i,k]} : yq[i,k,1] = 0;

